<?php
namespace DejwCake\StandardAuth\Controller\Admin;

use Cake\Event\Event;
use Cake\I18n\I18n;
use Cake\Log\Log;

/**
 * Roles Controller
 *
 * @property \DejwCake\StandardAuth\Model\Table\RolesTable $Roles
 */
class RolesController extends AppController
{

    /**
     * Before filter callback.
     *
     * @param \Cake\Event\Event $event The beforeRender event.
     * @return \Cake\Network\Response|null|void
     */
    public function beforeFilter(Event $event)
    {
        parent::beforeFilter($event);
    }

    /**
     * Check if the provided user is authorized for the request.
     *
     * @param array|\ArrayAccess|null $user The user to check the authorization of.
     *   If empty the user fetched from storage will be used.
     * @param \Cake\Network\Request|null $request The request to authenticate for.
     *   If empty, the current request will be used.
     * @return bool True if $user is authorized, otherwise false
     */
    public function isAuthorized($user = null) {
        if($user->hasRole('superadmin')) {
            return true;
        }
        return parent::isAuthorized($user);;
    }

    /**
     * Index method
     *
     * @return \Cake\Network\Response|null
     */
    public function index()
    {
        $roles = $this->Roles->find('all');
        $this->set(compact('roles'));
        $this->set('_serialize', ['roles']);
    }

    /**
     * View method
     *
     * @param string|null $id Role id.
     * @return \Cake\Network\Response|null
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
        $role = $this->Roles->find('translations', [
            'contain' => ['Users']
        ])->where(['id' => $id])->firstOrFail();

        $this->set('role', $role);
        $this->set('_serialize', ['role']);
    }

    /**
     * Add method
     *
     * @return \Cake\Network\Response|void Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $role = $this->Roles->newEntity();
        if ($this->request->is('post')) {
            $role = $this->Roles->patchEntity($role, $this->request->data, [
                'translations' => true
            ]);
            if ($this->Roles->save($role)) {
                $this->Flash->success(__d('dejw_cake_standard_auth', 'The role has been saved.'));

                return $this->redirect(['action' => 'index']);
            } else {
                Log::error('Entity could not be saved. Entity: '.var_export($role, true));
                $this->Flash->error(__d('dejw_cake_standard_auth', 'The role could not be saved. Please, try again.'));
            }
        }
        $users = $this->Roles->Users->find('list', ['limit' => 200]);
        $this->set(compact('role', 'users'));
        $this->set('_serialize', ['role']);
    }

    /**
     * Edit method
     *
     * @param string|null $id Role id.
     * @return \Cake\Network\Response|void Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Network\Exception\NotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $role = $this->Roles->find('translations', [
            'contain' => ['Users']
        ])->where(['id' => $id])->firstOrFail();
        if ($this->request->is(['patch', 'post', 'put'])) {
            $role = $this->Roles->patchEntity($role, $this->request->data, [
                'translations' => true
            ]);
            if ($this->Roles->save($role)) {
                $this->Flash->success(__d('dejw_cake_standard_auth', 'The role has been saved.'));

                return $this->redirect(['action' => 'index']);
            } else {
                Log::error('Entity could not be saved. Entity: '.var_export($role, true));
                $this->Flash->error(__d('dejw_cake_standard_auth', 'The role could not be saved. Please, try again.'));
            }
        }
        $users = $this->Roles->Users->find('list', ['limit' => 200]);
        $this->set(compact('role', 'users'));
        $this->set('_serialize', ['role']);
    }

    /**
     * Delete method
     *
     * @param string|null $id Role id.
     * @return \Cake\Network\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
        $this->request->allowMethod(['post', 'delete']);
        $role = $this->Roles->get($id);
        if ($this->Roles->delete($role)) {
            $this->Flash->success(__d('dejw_cake_standard_auth', 'The role has been deleted.'));
        } else {
            $this->Flash->error(__d('dejw_cake_standard_auth', 'The role could not be deleted. Please, try again.'));
        }

        return $this->redirect(['action' => 'index']);
    }

    /**
     * Enable method
     *
     * @param string|null $id Role id.
     * @return \Cake\Network\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function enable($id = null)
    {
        $this->request->allowMethod(['post']);
        $role = $this->Roles->get($id);

        $role->changeEnableStatus();
        if ($this->Roles->save($role)) {
            $this->Flash->success(__d('dejw_cake_standard_auth', 'The role status has been changed.'));
        } else {
            $this->Flash->error(__d('dejw_cake_standard_auth', 'The role status could not be changed. Please, try again.'));
        }

        return $this->redirect(['action' => 'index']);
    }
}
